@@ -1,10 +1,19 @@
 from __future__ import annotations
 
-from dataclasses import dataclass, field, replace
+from dataclasses import dataclass, field
 from enum import Enum
-from typing import Any, Dict, Generator, Iterable, List, Optional, Set
+from typing import Any, Dict, Generator, Iterable, List, Optional, Set, Union
 
-from .properties import EnumProperty, ListProperty, Property, RefProperty, property_from_dict
+from .properties import (
+    DateProperty,
+    DateTimeProperty,
+    EnumListProperty,
+    EnumProperty,
+    Property,
+    ReferenceListProperty,
+    RefProperty,
+    property_from_dict,
+)
 from .reference import Reference
 from .responses import ListRefResponse, RefResponse, Response, response_from_dict
 
@@ -91,7 +100,7 @@ class Endpoint:
             self.relative_imports.add(import_string_from_reference(self.form_body_reference, prefix="..models"))
         if (
             self.json_body is not None
-            and isinstance(self.json_body, (ListProperty, RefProperty, EnumProperty))
+            and isinstance(self.json_body, (ReferenceListProperty, EnumListProperty, RefProperty, EnumProperty))
             and self.json_body.reference is not None
         ):
             self.relative_imports.add(import_string_from_reference(self.json_body.reference, prefix="..models"))
@@ -108,8 +117,16 @@ class Endpoint:
             prop = property_from_dict(
                 name=param_dict["name"], required=param_dict["required"], data=param_dict["schema"]
             )
-            if isinstance(prop, (ListProperty, RefProperty, EnumProperty)) and prop.reference:
+            if isinstance(prop, DateProperty):
+                self.relative_imports.add("from datetime import date")
+            elif isinstance(prop, DateTimeProperty):
+                self.relative_imports.add("from datetime import datetime")
+            elif (
+                isinstance(prop, (ReferenceListProperty, EnumListProperty, RefProperty, EnumProperty))
+                and prop.reference
+            ):
                 self.relative_imports.add(import_string_from_reference(prop.reference, prefix="..models"))
+
             if param_dict["in"] == ParameterLocation.QUERY:
                 self.query_parameters.append(prop)
             elif param_dict["in"] == ParameterLocation.PATH:
@@ -151,24 +168,36 @@ class Schema:
     relative_imports: Set[str]
 
     @staticmethod
-    def from_dict(d: Dict[str, Any], /) -> Schema:
-        """ A single Schema from its dict representation """
+    def from_dict(d: Dict[str, Any], /, name: str) -> Schema:
+        """ A single Schema from its dict representation
+        :param d:    Dict representation of the schema
+        :param name: Name by which the schema is referenced, such as a model name.  Used to infer the type name if a `title` property is not available.
+        """
         required_set = set(d.get("required", []))
         required_properties: List[Property] = []
         optional_properties: List[Property] = []
         relative_imports: Set[str] = set()
 
-        for key, value in d["properties"].items():
+        ref = Reference.from_ref(d.get("title", name))
+
+        for key, value in d.get("properties", {}).items():
             required = key in required_set
             p = property_from_dict(name=key, required=required, data=value)
             if required:
                 required_properties.append(p)
             else:
                 optional_properties.append(p)
-            if isinstance(p, (ListProperty, RefProperty, EnumProperty)) and p.reference:
-                relative_imports.add(import_string_from_reference(p.reference))
+            if isinstance(p, DateTimeProperty):
+                relative_imports.add("from datetime import datetime")
+            elif isinstance(p, DateProperty):
+                relative_imports.add("from datetime import date")
+            elif isinstance(p, (ReferenceListProperty, EnumListProperty, RefProperty, EnumProperty)) and p.reference:
+                # don't add an import for self-referencing schemas
+                if p.reference.class_name != ref.class_name:
+                    relative_imports.add(import_string_from_reference(p.reference))
+
         schema = Schema(
-            reference=Reference.from_ref(d["title"]),
+            reference=ref,
             required_properties=required_properties,
             optional_properties=optional_properties,
             relative_imports=relative_imports,
@@ -180,32 +209,34 @@ class Schema:
     def dict(d: Dict[str, Dict[str, Any]], /) -> Dict[str, Schema]:
         """ Get a list of Schemas from an OpenAPI dict """
         result = {}
-        for data in d.values():
-            s = Schema.from_dict(data)
+        for name, data in d.items():
+            s = Schema.from_dict(data, name=name)
             result[s.reference.class_name] = s
         return result
 
 
 @dataclass
 class OpenAPI:
-    """ Top level OpenAPI spec """
+    """ Top level OpenAPI document """
 
     title: str
-    description: str
+    description: Optional[str]
     version: str
     schemas: Dict[str, Schema]
     endpoint_collections_by_tag: Dict[str, EndpointCollection]
-    enums: Dict[str, EnumProperty]
+    enums: Dict[str, Union[EnumProperty, EnumListProperty]]
 
     @staticmethod
-    def _check_enums(schemas: Iterable[Schema], collections: Iterable[EndpointCollection]) -> Dict[str, EnumProperty]:
+    def _check_enums(
+        schemas: Iterable[Schema], collections: Iterable[EndpointCollection]
+    ) -> Dict[str, Union[EnumProperty, EnumListProperty]]:
         """
         Create EnumProperties for every enum in any schema or collection.
         Enums are deduplicated by class name.
 
         :raises AssertionError: if two Enums with the same name but different values are detected
         """
-        enums: Dict[str, EnumProperty] = {}
+        enums: Dict[str, Union[EnumProperty, EnumListProperty]] = {}
 
         def _iterate_properties() -> Generator[Property, None, None]:
             for schema in schemas:
@@ -217,7 +248,7 @@ class OpenAPI:
                     yield from endpoint.query_parameters
 
         for prop in _iterate_properties():
-            if not isinstance(prop, EnumProperty):
+            if not isinstance(prop, (EnumProperty, EnumListProperty)):
                 continue
 
             if prop.reference.class_name in enums:
@@ -238,7 +269,7 @@ class OpenAPI:
 
         return OpenAPI(
             title=d["info"]["title"],
-            description=d["info"]["description"],
+            description=d["info"].get("description"),
             version=d["info"]["version"],
             endpoint_collections_by_tag=endpoint_collections_by_tag,
             schemas=schemas,
